#!/usr/bin/perl

use strict;
use warnings;
use XML::Simple;
use LWP::Simple;
use Getopt::Long;

my %opt;
GetOptions(
  'apikey=s'                          => \$opt{'apikey'},
  'limit=s'                           => \$opt{'limit'},
  'stop-on-error'                     => \$opt{'stop-on-error'},
) or die "bad usage";

die "usage: rank-weblogs.pl --apikey APIKEY [--limit N] [--stop-on-error] ".
        " < LIST_OF_URLS\n"
        unless $opt{'apikey'};


$|++;
my $count_errors = 0;
my $count_empty = 0;
my $results = { };
my $limit = $opt{'limit'};
my $now = time;
while (<>) {
  chomp;

  lookup_weblog($_)
    or (sleep 1 and lookup_weblog($_))
    or (sleep 1 and lookup_weblog($_));       # retries ;)

  sleep 1;

  if (defined($limit)) { last if ($limit-- == 0); }
  last if ($opt{'stop-on-error'} && $count_errors);
}
exit;

sub lookup_weblog {
  my $origurl = shift;

  my $url = 'http://api.technorati.com/bloginfo?key='.$opt{'apikey'}.'&url='.$origurl;
  warn "lookup: $origurl\n";
  my $doc = get($url);

  if (!defined $doc) {
    print "url='$origurl' error=1 last=$now\n";
    $count_errors++;
    return;
  }
  elsif ($doc eq '') {
    # this is intermittent, and often works on next lookup...
    print "url='$origurl' error=1 last=$now\n";
    $count_errors++;
    return;
  }

  my $xml = XMLin($doc);
  # print "got: $doc\n";
  # use Data::Dumper; warn "JMD ".Dumper($xml);

  # <!-- generator="Technorati API version 1.0 /bloginfo" -->
  # <tapi version="1.0"> <document> <result> <url>http://taint.org</url>
  # <weblog> <name> taint.org: Happy Software Prole</name>
  # <url>http://taint.org</url>
  # <rssurl>http://taint.org/index.xml</rssurl>
  # <inboundblogs>112</inboundblogs>
  # <inboundlinks>765</inboundlinks>
  # <lastupdate>2005-06-13 20:46:52 GMT</lastupdate>
  # <rank>26350</rank>
  # <lat>33.646059</lat>
  # <lon>-117.84237</lon>
  # <foafurl>http://jmason.org/gems/my-foaf.xrdf</foafurl>
  # </weblog>
  # <inboundblogs>112</inboundblogs>
  # <inboundlinks>765</inboundlinks>
  # </result> </document> </tapi>

  if (!$xml->{document}->{result}->{weblog}) {
    warn "odd XML for $url\n";
    print "url='$origurl' error=1 last=$now\n";
    $count_errors++;
    return;
  }

  if (!$xml->{document}->{result}->{weblog}->{url}) {
    # it exists, but is empty; no URL registered.  odd
    warn "odd XML for $url\n";
    print "url='$origurl' error=1 last=$now\n";
    $count_errors++;
    return;
  }

  if (ref $xml->{document}->{result}->{weblog}->{rank} eq 'HASH') {
    # empty tag: <rank/>
    # this consistently seems to mean "unregistered site"
    warn "empty rank tag for $url\n";
    print "url='$origurl' unreg=1 last=$now\n";
    $count_empty++;
    return 1;
  }

  print "url='$origurl'".
      " inboundblogs=".$xml->{document}->{result}->{weblog}->{inboundblogs}.
      " inboundlinks=".$xml->{document}->{result}->{weblog}->{inboundlinks}.
      " rank=".$xml->{document}->{result}->{weblog}->{rank}.
      " last=$now\n";
  return 1;
}

