#!/usr/bin/perl

use strict; use warnings;

my $html = 1;           # 0 for debug, text-only mode
my $full = 0;
if ($ARGV[0] && $ARGV[0] eq '-f') { $full = 1; shift @ARGV; }

my $c_error = 0;
my $c_unreg = 0;
my $c_reg   = 0;
my %res = ();

while (<>) {
  s/^url='(.*?)'//;
  my $url = $1;

  my %p = ();
  foreach my $pair (split ' ', $_) {
    $pair =~ /^(.*?)=(.*)$/ or next;
    $p{$1} = $2;
  }

  if ($p{unreg}) { $c_unreg++; next; }
  if ($p{error}) { $c_error++; next; }

  if (!$p{rank}) {
    $c_error++; next;
  }

  $c_reg++;
  $res{$url} = { };
  $res{$url}->{inboundblogs} = $p{inboundblogs} + 0;
  $res{$url}->{inboundlinks} = $p{inboundlinks} + 0;
  $res{$url}->{rank} = $p{rank} + 0;
}

my $reporttype = ($full ? "List" : "Top 100");

print qq{
  <h1> Blog ${reporttype}s By Technorati Rank </h1>
  <ul>
    <li>Generated at: }.`date`.qq{ </li>
    <li>unregistered blogs pinging planet: $c_unreg </li>
    <li>registered blogs found in Technorati: $c_reg </li>
    <li>errors parsing: $c_error </li>
  </ul>

  <h2> Index </h2>
  <ul>
    <li><a href="index.html#byrank">Top 100 by rank / inbound blog links</a></li>
    <li><a href="index.html#bylinks">Top 100 by inbound links</a></li>
    <li><a href="full.html#byrank">Full list by rank / inbound blog links</a></li>
    <li><a href="full.html#bylinks">Full list by inbound links</a></li>
  </ul>
};

print "<a name='byrank'></a><h2>$reporttype by rank / inbound blog links:</h2>\n";
gen_ranking_table (100, 'rank');

print "<a name='bylinks'></a><h2>$reporttype by inbound links:</h2>\n";
gen_ranking_table (100, 'inboundlinks', 1);

sub gen_ranking_table {
  my $limit = shift;
  my $hash = shift;
  my $rev = shift;
  my $idx = 0;

  if ($html) {
    print qq{
      <table><tr>
      <th> Position</th>
      <th> Rank </th>
      <th> Inbound blogs </th>
      <th> Inbound links </th>
      <th> Blog </th>
      </tr>
    };
  }

  # sort primarily by the chosen key, then fallback to all others
  # to settle even matches
  my @sorted = sort {
          ($res{$rev?$b:$a}->{$hash}+0)
                <=> ($res{$rev?$a:$b}->{$hash}+0) or
          ($res{$rev?$a:$b}->{rank}+0)
                <=> ($res{$rev?$b:$a}->{rank}+0) or
          ($res{$rev?$a:$b}->{inboundblogs}+0)
                <=> ($res{$rev?$b:$a}->{inboundblogs}+0) or
          ($res{$rev?$a:$b}->{inboundlinks}+0)
                <=> ($res{$rev?$b:$a}->{inboundlinks}+0)
        } keys %res;

  foreach my $url (@sorted) {
    $idx++;

    if ($html) {
      print qq{

      <tr>
      <td align='right'><b> $idx </b></td>
      <td align='right'> $res{$url}->{rank} </td>
      <td align='right'> $res{$url}->{inboundblogs} </td>
      <td align='right'> $res{$url}->{inboundlinks} </td>
      <td> &nbsp; <a href='$url'>$url</a> 
        <a href='http://technorati.com/blogs/$url'><img border='0'
        src='http://static.technorati.com/static/css/img/icn/talkbubble.png'
        /></a> </td>
      </tr>

      };
    } else {

      printf "%3d. %8d %5d %5d   %s\n", $idx,
          $res{$url}->{rank}, $res{$url}->{inboundblogs},
          $res{$url}->{inboundlinks}, $url;

    }

    last if (!$full && $idx >= $limit);
  }

  if ($html) {
    print qq{
      </table>
    };
  }

}

__DATA__

to update on site:
 
  zipc top100code.tgz README *.pl
  scptaint top100code.tgz

